// KDPatch.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "KDPatch.h"
#include <strsafe.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*////////////////////////////////////////////////////////////////////

	KDPatch is called from Khan Desktop to replace its in use exe.

	It is called from CDoc::UpdateApplication() via ShellExecuteExW() 
	which has its lpVerb set to "runas".

	The objective is launch this app as a seperate proccess requiring
	admin rights, and have Vista and Win7's UAC accept the idea.

	KDPatch is manifested to require admin rights.

*/////////////////////////////////////////////////////////////////////

/*
	Error returns:
	1 - Module handle
	2 - Not admin	
	3 - Invalid number of parameters
	4 - Invalid update exe path
	5 - Invalid program exe path
	6 - End of file reached
*/

// See also SHFILEOPSTRUCT SHFileOperation

///////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

// Note params containing spaces must be in quotes.
int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	CString str;

	// Initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		return 1;
	}
	else
	{
		// Ensure admin, else MoveFileEx() will rename Khan.exe 
		// before it fails (crazy and absurd).
		if(!IsAdmin())
		{
			str = _T("ERROR: Not a member of the Administrators local group.");
			wcout << (LPCTSTR)str << endl;
			//AfxMessageBox(_T("Update failed - You are not a member of the Administrators local group."));
			return 2;
		}

		// Print args.		
		for(int i = 0; i < argc; i++)
		{
			str.Format(_T("Arg # %d"), i);
			wcout << (LPCTSTR)str << endl;
			str = argv[i];
			wcout << (LPCTSTR)str << endl;
		}

		// Ensure 4 args.
		if(argc < 4)
		{
			//AfxMessageBox(_T("Update failed - invalid number of parameters."));
			str = _T("ERROR: Invalid number of parameters.");
			wcout << (LPCTSTR)str << endl;
			return 3;		
		}

		// Paths.
		CString command = argv[0];			// run command for this app
		CString exePath = argv[1];			// proccess dir/Khan.exe
		CString exePathOld = argv[2];		// proccess dir/Khan.exe.old
		CString downloadPath = argv[3];		// app data folder/Khan_new.exe

		// Valid download path. Note path lengths must be < MAX_PATH.
		CFileStatus status;
		if(!CFile::GetStatus(downloadPath, status)) // static version for test.
		{
			// Messsage user.
			//AfxMessageBox(_T("Update failed - Invalid update exe path."));
			str = _T("ERROR: Invalid number of parameters.");
			wcout << (LPCTSTR)str << endl;
			return 4;
		}
		str = _T("Download path valid.");
		wcout << (LPCTSTR)str << endl;

		// Valid exe path.
		if(!CFile::GetStatus(exePath, status)) // static version for test.
		{
			// Messsage user.
			//AfxMessageBox(_T("Update failed - invalid program exe path."));
			str = _T("ERROR: Invalid program exe path.");
			wcout << (LPCTSTR)str << endl;
			return 5;
		}
		str = _T("Exe path valid. Atempting to rename exe");
		wcout << (LPCTSTR)str << endl;		

		// MoveFileEx flags.
		DWORD flags = MOVEFILE_COPY_ALLOWED | MOVEFILE_REPLACE_EXISTING;

		// Swap exe.
		//
		// Move in-use exe to old (rename).
		if(MoveFileEx(exePath, exePathOld, flags))
		{
			str = _T("Exe successfully renamed");
			wcout << (LPCTSTR)str << endl;

			// Move downloaded exe to in-use exe.
			if(MoveFileEx(downloadPath, exePath, flags))
			{
				// Note the downloaded file has been moved and deleted..
				str = _T("Exe successfully replaced with download");
				wcout << (LPCTSTR)str << endl;
				return 0;
			}
			// MoveFileEx Error.
			else
			{
				return ErrorMessage();
			}
		}
		// MoveFileEx Error
		else
		{
			return ErrorMessage();
		}
	}

	return 6;
}

BOOL IsAdmin()
{
	//return IsUserAnAdmin();

/*	From: msdn CheckTokenMembership

	Routine Description: This routine returns TRUE if the caller's
	process is a member of the Administrators local group. Caller is NOT
	expected to be impersonating anyone and is expected to be able to
	open its own process and process token. 
	Arguments: None. 
	Return Value: 
	TRUE - Caller has Administrators local group. 
	FALSE - Caller does not have Administrators local group. --
*/ 
	SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
	PSID AdministratorsGroup; 
	BOOL b = AllocateAndInitializeSid(&NtAuthority,
									  2,
									  SECURITY_BUILTIN_DOMAIN_RID,
									  DOMAIN_ALIAS_RID_ADMINS,
									  0, 0, 0, 0, 0, 0,
									  &AdministratorsGroup); 
	if(b) 
	{
		if(!CheckTokenMembership( NULL, AdministratorsGroup, &b)) 
		{
			b = FALSE;
		} 
		FreeSid(AdministratorsGroup); 
	}
	return b;
}

// Get the MoveFileEx Error mesage.
int ErrorMessage()
{ 
	LPVOID lpMessageBuffer;

	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
				  FORMAT_MESSAGE_FROM_SYSTEM,
				  NULL,
				  GetLastError(),
				  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), //The user default language
				  (LPTSTR) &lpMessageBuffer,
				  0,
				  NULL );

	// Display string in console.
	CString err;
	err.Format(_T("KDPatch error: %s"), lpMessageBuffer);
	wcout << (LPCTSTR)err << endl;

	// Free the buffer allocated by the system
	LocalFree(lpMessageBuffer);

	// Show message box.
	AfxMessageBox(err);
	return 7;

}


