
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// CKhanDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "KhanDialog.h"
#include "KhanInet.h"
#include "Doc.h"
#include "MainFrm.h"
#include "ContentsBackupDlg.h"
#include "ContentsRestoreDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define LINESIZE 8

/////////////////////////////////////////////////////////////////////////////
// CKhanDialog dialog


CKhanDialog::CKhanDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CKhanDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CKhanDialog)
	m_strEdit = _T("");
	m_bSortChild = FALSE;
	m_bSortRoot = FALSE;
	//}}AFX_DATA_INIT

	// Set bkBrush
	m_bkBrush.CreateSolidBrush(RGB(255, 255, 255));

	// Scrolling
	m_nDlgHeight	= 0;	// Workspace height in pixels
	m_nVScrollPos	= 0;	// Vertical scroll position
	m_nVPageSize	= 0;	// Vertical page size
}


void CKhanDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CKhanDialog)
	DDX_Control(pDX, IDC_RESTORE_CONTENTS, m_ctrlRestoreContents);
	DDX_Control(pDX, IDC_BACKUP_CONTENTS, m_ctrlBackupContents);
	DDX_Control(pDX, IDC_KHAN_DATA_EDIT, m_ctrlEdit);
	DDX_Control(pDX, IDC_BUTTON_UPDATE, m_ctrlUpdateBtn);
	DDX_Control(pDX, IDC_KHAN_DATA_PROGRESS, m_ctrlProgress);
	DDX_Text(pDX, IDC_KHAN_DATA_EDIT, m_strEdit);
	DDX_Control(pDX, IDC_STATIC_KBPS, m_ctrlStaticKBps);	
	DDX_Check(pDX, IDC_CHECK_SORT_CHILD, m_bSortChild);
	DDX_Check(pDX, IDC_CHECK_SORT_ROOT, m_bSortRoot);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CKhanDialog, CDialog)
	//{{AFX_MSG_MAP(CKhanDialog)
	ON_BN_CLICKED(IDC_BUTTON_UPDATE, OnButtonUpdate)
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_CHECK_SORT_CHILD, OnCheckSortChild)
	ON_BN_CLICKED(IDC_CHECK_SORT_ROOT,	OnCheckSortRoot)
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_BACKUP_CONTENTS, OnContentsBackup)
	ON_BN_CLICKED(IDC_RESTORE_CONTENTS, OnContentsRestore)
	//}}AFX_MSG_MAP
	ON_MESSAGE (WM_USER_STATUS_STRING,	OnStatusString)
	ON_MESSAGE (WM_USER_PROGRESS_POS,	OnProgressPos)
	ON_MESSAGE (WM_USER_PROGRESS_MAX,	OnProgressMax)
	ON_MESSAGE (WM_USER_FINISHED,		OnUpdateFinished)
	ON_MESSAGE (WM_USER_KBPS,			OnKBps)


END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKhanDialog message handlers

BOOL CKhanDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	m_ctrlUpdateBtn.EnableWindow(TRUE);
	m_strEdit = _T("Press the Update button to\r\nbegin.\r\n\r\n")
				_T("Updates are obtained from\r\n")
				_T("www.khanacademy.org.");
	m_ctrlProgress.SetRange32(0, 100);
	m_ctrlProgress.SetPos(0);

	// Sort checks.
	m_bSortRoot	 = TRUE;
	m_bSortChild = FALSE;

	// Set KBps string.
	m_ctrlStaticKBps.SetWindowText(_T("Download Rate (KBps) :  0.0"));

	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// Called from Worker Thread via SendMessage.
LRESULT CKhanDialog::OnStatusString(WPARAM wParam, LPARAM lParam)	// WM_USER_STATUS_STRING
{
	// Get message string from CApp.
	CString str;
	if(	((CApp*)AfxGetApp())->GetThreadMessage(str) )	// move to CMainFrame?
	{
		// Set the CEdit
		m_ctrlEdit.SetWindowText(str);
		m_ctrlEdit.LineScroll(m_ctrlEdit.GetLineCount());
		m_ctrlEdit.UpdateWindow();
	}
    return 0;
}

// Called with Dialog Handle from worker thread.
LRESULT CKhanDialog::OnProgressPos(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS_POS
{
	TRACE(_T("CKhanDialog OnProgressPos Pos = %d\n"), (int)lParam);

	m_ctrlProgress.SetPos((int)lParam);
    return 0;
}
// Called with Dialog Handle from worker thread.
LRESULT CKhanDialog::OnProgressMax(WPARAM wParam, LPARAM lParam)	// WM_USER_PROGRESS_MAX
{
	TRACE(_T("CKhanDialog OnProgressMax Max = %d\n"), (int)lParam);

	m_ctrlProgress.SetRange32(0, (int)lParam);
    return 0;
}

// KB per second.
// wparam is the seconds, lparam is a fraction
LRESULT CKhanDialog::OnKBps(WPARAM wParam, LPARAM lParam)		// WM_USER_KBPS
{
	double sec = (double)(int)wParam;
	double fract = (double)(int)lParam;
	fract /= 10;
	sec += fract;

	// Set static member str.
	CString s;
	s.Format(_T("Download Rate (KBps) :  %.1f"), sec);
	m_ctrlStaticKBps.SetWindowText(s);

//	TRACE(_T("KBps: %.1f\n"), sec);
    return 0;
}

LRESULT CKhanDialog::OnUpdateFinished(WPARAM wParam, LPARAM lParam)	// WM_USER_FINISHED
{
	m_ctrlUpdateBtn.SetWindowText(_T("Update"));
	m_ctrlProgress.SetPos(0);
	// Set KBps string.
	m_ctrlStaticKBps.SetWindowText(_T("Download Rate (KBps) :  0.0"));

	// Enable backup and restore
	m_ctrlBackupContents.EnableWindow(TRUE);
	m_ctrlRestoreContents.EnableWindow(TRUE);

	return 0;
}

void CKhanDialog::OnButtonUpdate() 
{
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// Get sort checks.
	UpdateData(TRUE);

	CString str;
	m_ctrlUpdateBtn.GetWindowText(str);

	// Update via CDoc,
	if(str == _T("Update"))
	{
		pDoc->UpdateKhanLibrary(GetSafeHwnd(),
								m_bSortRoot, m_bSortChild, FALSE);
		m_ctrlUpdateBtn.SetWindowText(_T("Cancel"));

		// Disable backup and restore
		m_ctrlBackupContents.EnableWindow(FALSE);
		m_ctrlRestoreContents.EnableWindow(FALSE);
	}

	// Cancel via CDoc,
	else if(str == _T("Cancel"))
	{
		pDoc->CancelUpdate();
	}
	return;
}

void CKhanDialog::OnCheckSortChild() 
{
	// TODO: Add your control notification handler code here
	
}

void CKhanDialog::OnCheckSortRoot() 
{
	// TODO: Add your control notification handler code here
	
}

void CKhanDialog::OnSize(UINT nType, int cx, int cy) 
{
	TRACE(_T("In CKhanDialog OnSize\n"));
	int nVScrollMax = 0;

	// Get dialog height from app - initialized in CTabView::CreatePages.
	m_nDlgHeight = ((CApp*)AfxGetApp())->m_updateDlgRect.Height();

	// Initialize and update our scrollinfo for vert bar.
	//
	// If dialog is larger than window, adjust bars
	if (cy < m_nDlgHeight) 
	{
		nVScrollMax = m_nDlgHeight - 1;
		m_nVPageSize = cy;
	}

	// Else, effectively hides bars
	else
	{
		// no scroll bars are needed. Position the dlg back to normal
		ScrollWindow(0, m_nVScrollPos);
		// init scrollinfo
		m_nVScrollPos = m_nVPageSize = 0;
	}

	SCROLLINFO si;
	si.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
	si.nMin = 0;
	si.nMax = nVScrollMax;
	si.nPos = m_nVScrollPos;
	si.nPage = m_nVPageSize;

	SetScrollInfo (SB_VERT, &si, TRUE);

	// dlg does not repaint after update. This forces the issue.
	Invalidate();
	if(si.nPos != 0)
		PostMessage(WM_VSCROLL);

	CDialog::OnSize(nType, cx, cy);		
}

void CKhanDialog::OnVScroll(UINT nCode, UINT nPos, CScrollBar* pScrollBar) 
{
    int nDelta;

    // Compute the vertical scroll distance, or "delta."
    switch (nCode) 
	{

    case SB_LINEUP:
        nDelta = -LINESIZE;
        break;

    case SB_PAGEUP:
        nDelta = -m_nVPageSize;
        break;

    case SB_THUMBTRACK:
        nDelta = (int) nPos - m_nVScrollPos;
        break;

    case SB_PAGEDOWN:
        nDelta = m_nVPageSize;
        break;

    case SB_LINEDOWN:
        nDelta = LINESIZE;
        break;

    default: // Ignore other scroll bar messages
        return;
    }

    // Adjust the delta if adding it to the current scroll position would
    // cause an underrun or overrun.
    int nScrollPos = m_nVScrollPos + nDelta;
    int nMaxPos = m_nDlgHeight - m_nVPageSize;

    if (nScrollPos < 0)
        nDelta = -m_nVScrollPos;
    else if (nScrollPos > nMaxPos)
        nDelta = nMaxPos - m_nVScrollPos;

    // Update the scroll position and scroll the window.
    if (nDelta != 0) 
	{
        m_nVScrollPos += nDelta;
        SetScrollPos (SB_VERT, m_nVScrollPos, TRUE);
        ScrollWindow(0, -nDelta);
    }
	CDialog::OnVScroll(nCode, nPos, pScrollBar);
}

// Ignore Return, Escape and Space keys.
BOOL CKhanDialog::PreTranslateMessage(MSG* pMsg) 
{
	// TODO: Add your specialized code here and/or call the base class
	
	if(pMsg->message == WM_KEYDOWN		&&
	   ( pMsg->wParam == VK_RETURN		|| 
		 pMsg->wParam == VK_ESCAPE		||
		 pMsg->wParam == VK_SPACE )		)
	{
		return TRUE;
	}
	
	return CDialog::PreTranslateMessage(pMsg);
}

// Backup Contents
void CKhanDialog::OnContentsBackup() 
{
	CContentsBackupDlg dlg;
	if(dlg.DoModal() == IDOK)
	{
		// CContentsBackupDlg does it all.
	}
	
}

// Restore Contents.
void CKhanDialog::OnContentsRestore() 
{
	CContentsRestoreDlg dlg;
	if(dlg.DoModal() == IDOK)
	{
		// CContentsRestoreDlg does it all.
	}
	
}
