// ListFiles.cpp : implementation file
//

#include "stdafx.h"
#include "khan.h"
#include "ListFiles.h"
#include "CopyFiles.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListFiles

CListFiles::CListFiles()
{
	m_lastColSort = 0;
	m_lastColAscend = FALSE;
}

CListFiles::~CListFiles()
{
	// Clean items array.
	m_itemAr.RemoveAll();

	// MFC destroys the CListCtrl.
}

BEGIN_MESSAGE_MAP(CListFiles, CListCtrl)
	//{{AFX_MSG_MAP(CListFiles)
	ON_NOTIFY_REFLECT(LVN_GETDISPINFO, OnGetdispinfo)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	ON_WM_CONTEXTMENU()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
//	ON_NOTIFY(NM_CUSTOMDRAW, IDC_MY_LIST, OnCustomDrawList )
//	ON_NOTIFY_REFLECT( NM_CUSTOMDRAW, OnCustomDrawList)
END_MESSAGE_MAP()


// Create the list oontrol and populate.
// Bypassing usual message map.
BOOL CListFiles::CreateList(CWnd * parent, CRect client)
{
	// Make sure.
	if(ListShowing())
		return FALSE;

	// Create. 
	if(!Create(WS_CHILD				|
			   WS_VISIBLE			|
			   LVS_OWNERDRAWFIXED	|	// fixed
			   LVS_REPORT,
			   client, 
			   parent, IDI_LISTFILES))
	{
		return FALSE;
	}

	// Get videos from storage.
	if(!LoadVideoFiles())
	{
		// If none, set an empty list.
		InsertColumn (0, _T("File Name"), LVCFMT_LEFT, 150);
		InsertColumn (1, _T("Khan Title"), LVCFMT_LEFT, 250);
		InsertColumn (2, _T("Size KB"), LVCFMT_LEFT, 100);
		return TRUE;
	}

	// Set list font to match the contents tab.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(pDoc)
	{
		// Clean any existing. Also done in DestroyList().
		// Note font member persists till program is closed.
		if((HFONT)m_listFont)
		{
			m_listFont.DeleteObject();
		}
		// Set font - slightly smaller.
		LOGFONT lf = pDoc->m_treeLf;
	//	lf.lfHeight += lf.lfHeight < 0 ? 2 : -2;
		m_listFont.CreateFontIndirect(&lf);
		SetFont(&m_listFont);
	}


	// Get col widths based on text extent.
	int c1, c2, c3;
	GetColWidths(c1, c2, c3);

	// Add columns.
    InsertColumn (0, _T("File Name"), LVCFMT_LEFT, c1 + 10);
    InsertColumn (1, _T("Khan Title"), LVCFMT_LEFT, c2 + 10);
    InsertColumn (2, _T("Size KB"), LVCFMT_LEFT, c3 + 10);

	// Add video files.
	SetFilesList();

	// Set cursel video, if any.
	SetCurSelVideo();

	return TRUE;
}

// Destroy the list.
// Bypassing usual message map.
void CListFiles::DestroyList()
{
	TRACE(_T("In CListFiles DestroyList.\n"));

	if(ListShowing())
	{
		// Clean up.
		//
		// Clean CListCtrl items.
		DeleteAllItems();

		// Empty item array.
		m_itemAr.RemoveAll();

		// Clean font.
		if((HFONT)m_listFont)
			m_listFont.DeleteObject();

		// Destroy CListCtrl.
		DestroyWindow();
	}
}

// Get 3 col widths based on string lengths.
void CListFiles::GetColWidths(int & col1, int & col2, int & col3)
{
	col1 = 150;
	col2 = 250;
	col3 = 100;
	int w, sz = m_itemAr.GetSize();
	CString s;

	for(int i = 0; i < sz; i ++)
	{
		// Name.
		w = GetStringWidth(m_itemAr[i].m_fileName);
		if(w > col1)
			col1 = w;

		// Title.
		w = GetStringWidth(m_itemAr[i].m_title);
		if(w > col2)
			col2 = w;

		// Size kb.
		s.Format(_T("%d"), m_itemAr[i].m_sizeKb);
		w = GetStringWidth(s);
		if(w > col3)
			col3 = w;
	}

	// Limit title widths to approx MAX_TITLE_LEN chars.
	// See also CCopyFiles::OnButtonCopy().
//	s = _T("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
//	w = GetStringWidth(s) * MAX_TITLE_LEN / 52;
//	if(w < col2)
//		col2 = w;
	if(col2 > col1 * 2)
		col2 = col1 * 2;
}

// Get videos as CListItems.
BOOL CListFiles::LoadVideoFiles() 
{
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return FALSE;
	CTopicTree & tree = pDoc->GetTree();

	// Prepare item Array.
	m_itemAr.RemoveAll();
	m_itemAr.SetSize(0, 512);

	// Vars.
	CVideo video;
	CListItem item;
	int szKb;

	// File vars.
	WIN32_FIND_DATA	FindData;
	ULARGE_INTEGER	fileSz;
	HANDLE	hFiles;
	CString fileName;

	// Get video store directory.
	CString vidStore = pDoc->m_videoStorePath;
	vidStore += _T("\\");
	CString dir = vidStore;
	dir += _T("*");

	// Get a CVideo for each file.
	if((hFiles = FindFirstFile(dir, &FindData)) != INVALID_HANDLE_VALUE)
	{
		while(FindNextFile(hFiles, &FindData))
		{
			// Watch for this or other directories.
			if(FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				continue;

			// Get the file name.
			fileName = FindData.cFileName;

			// Ensure an mp4.
			if(fileName.Find(_T("mp4"), fileName.GetLength() - 3) == -1)
				continue;

			// If not in the db.
			if(!tree.VideoFileUsed(fileName, &video))
				continue;

			// Get file size;
			fileSz.LowPart = FindData.nFileSizeLow; 
			fileSz.HighPart = FindData.nFileSizeHigh; 
			szKb = (int)(_int64)fileSz.QuadPart / 1024;


			// Set the item.
			item.m_fileName = fileName;
			item.m_title = video.title;
			item.m_sizeKb = szKb;

			// Add to array.
			m_itemAr.Add(item);
		}
		FindClose(hFiles);
	}
	// No video storage
	else
	{
//		m_strStaticInfo = _T("No video storage found.\nCannot set Titles.");
//		m_ctrlSetTitles.EnableWindow(FALSE);
//		UpdateData(FALSE);
		return FALSE;
	}

	// Get count.
	int nItems = m_itemAr.GetSize();

	// If no videos.
	if(!nItems)
	{
//		m_strStaticInfo = _T("Video Storage contains no video files.\nCannot set Titles.");
//		m_ctrlSetTitles.EnableWindow(FALSE);
//		UpdateData(FALSE);
		return FALSE;
	}
	
	return TRUE;
}

void CListFiles::SetFilesList()
{
	DeleteAllItems();

	int sz = m_itemAr.GetSize();
	LV_ITEM lvi;
	for(int i = 0; i < sz; i++)
	{
		ZeroMemory(&lvi, sizeof(LV_ITEM));
		lvi.mask = LVIF_TEXT;// | LVIF_STATE; // | LVIF_STATE test
		lvi.iItem = i; 
		lvi.iSubItem = 0; 
		lvi.iImage = 0;
		lvi.pszText = LPSTR_TEXTCALLBACK;

		if(InsertItem(&lvi) == -1)
			return;
	}
}

// Check if List has been created.
BOOL CListFiles::ListShowing()
{
	if(m_hWnd)
		return TRUE;
	else
		return FALSE;
}

// Resize.
void CListFiles::Resize(CRect rect)
{
	// Ensure.
	if(!ListShowing())
		return;
	if(rect.IsRectEmpty())
		return;

	// Resize.
	MoveWindow(&rect);
}

// Select video if currently selected in the Contents Tab.
void CListFiles::SetCurSelVideo()
{
	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;
	CTopicTree & tree = pDoc->GetTree();

	// If a video selected.
	if(pDoc->m_hCurVideo)
	{
		int index = -1;
		CVideo vid = pDoc->m_curVideo;

		if(vid.title.IsEmpty())
			return;

		// Search whole list.
		int sz = m_itemAr.GetSize();
		for(int i = 0; i < sz; i++)
		{
			if(m_itemAr[i].m_title == vid.title)
			{
				index = i;
				break;
			}
		}
		// If found, selects and scroll into view.
		if(index != -1)
		{
			// Desect any currently selected items.
			int i;
			POSITION pos = GetFirstSelectedItemPosition();
			while(pos)
			{
				i = GetNextSelectedItem(pos);
				SetItemState(i, 0, LVIS_SELECTED);
			}
			// Select the current video item
			SetItemState(index, LVIS_SELECTED, LVIS_SELECTED);// | LVIS_FOCUSED);
			EnsureVisible(index, TRUE);

			// Set focus to list.
			SetFocus();
		}
	}	
}


/////////////////////////////////////////////////////////////////////////////
// CListFiles message handlers

void CListFiles::OnGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*) pNMHDR;
	CString str;

    if (pDispInfo->item.mask & LVIF_TEXT) 
	{
		// Validate index.
		int index = pDispInfo->item.iItem;
		int sz = m_itemAr.GetSize();
		if(index < 0 || index >= sz)
		{
			TRACE(_T("ERROR CListFiles OnGetdispinfo array bounds.\n"));
			*pResult = 0;
			return;
		}
		CListItem & item = m_itemAr[index];

        switch (pDispInfo->item.iSubItem) 
		{

        case 0: // File name
			StringCbCopy(pDispInfo->item.pszText, 
						 (item.m_fileName.GetLength() + 1) * sizeof(TCHAR), 
						 item.m_fileName);
			//::lstrcpy(pDispInfo->item.pszText, item.m_fileName);
            break;

        case 1: // Title
			StringCbCopy(pDispInfo->item.pszText, 
						 (item.m_title.GetLength() + 1) * sizeof(TCHAR), 
						 item.m_title);
			//::lstrcpy(pDispInfo->item.pszText, item.m_title);
            break;

        case 2: // File size
			str.Format(_T("%d"), item.m_sizeKb);
			StringCbCopy(pDispInfo->item.pszText, 
						 (str.GetLength() + 1) * sizeof(TCHAR), 
						 str);
			//::lstrcpy(pDispInfo->item.pszText, str);
           break;
        }
    }
	else
	{
		int g = 0;
	}
	*pResult = 0;
}

// Column header clicked.
void CListFiles::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW * pNMListView = (NM_LISTVIEW*) pNMHDR;

	*pResult = 0;

	// iSubItem is the column clicked.
	int col = pNMListView->iSubItem;

	// Toggle.
	BOOL ascend = TRUE;
	if(col == m_lastColSort)
	{
		m_lastColAscend = !m_lastColAscend;
		ascend = m_lastColAscend;
	}

	// Sort with CItemArray sort functions.
	// rodo - save last col clicked, sort descending if reclicked.
	//
	// Sort File Name.
	if(col == 0)
		m_itemAr.SortNames(ascend);
	// Sort Titles.
	else if(col == 1)
		m_itemAr.SortTitles(ascend);
	// Sort Size.
	else if(col == 2)
		m_itemAr.SortSizes(ascend);
	// Bad column.
	else
		return;

	// Rebuild list.
	SetFilesList();

	// Save last.
	m_lastColSort = col;
	m_lastColAscend = ascend;
}

// Handle context.
void CListFiles::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	TRACE(_T("Entering CListFiles OnContextMenu\n"));

	// Get Doc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	int index = 0;
	CString str;
	CItemArray itemsToCopy;
	CListItem item;

	// Create the context.
	CMenu menu;
	menu.CreatePopupMenu();

	// Get selected items, if any.
	//
	// Note OnRButtonDown removes all selections if
	// user right clicked on a non selected area.
	int sz = m_itemAr.GetSize();
	POSITION pos = GetFirstSelectedItemPosition();
	while(pos)
	{
		index = GetNextSelectedItem(pos);

		// Be sure.
		if(index >= sz || index < 0)
		{
			TRACE(_T("ERROR CListFiles OnContextMenu out of bounds.\n"));
			continue;
		}

		// Copy item - indices will match.
		item = m_itemAr[index];
		itemsToCopy.Add(item);
	}
	// Add menu items.
	//
	// If selections.
	sz = itemsToCopy.GetSize();
	if(sz)
	{
		// Add menu item - copy files.
		if(sz > 1)
			str.Format(_T("Copy %d Videos To..."), sz);
		else
			str = _T("Copy Video To...");
		menu.AppendMenu(MF_STRING, ID_CTX_COPY_FILES, str);

		// Add menu item - delete files.
		if(sz >1)
			str.Format(_T("Delete %d Videos"), sz);
		else
			str = _T("Delete Video");
		menu.AppendMenu(MF_STRING, ID_CTX_DELETE_FILES, str);	
	}

	// Add default menu item - close the ListFiles window.	
	str = _T("Close List Video Files");
	menu.AppendMenu(MF_STRING, ID_CTX_CLOSE_FILES, str);

	// Ensure.
	if(menu.GetMenuItemCount() == 0)
		return;

	//////////////////////
	// Show Context Menu.
	int nCmd = menu.TrackPopupMenu(	TPM_LEFTALIGN  | 
									TPM_LEFTBUTTON |
									TPM_RETURNCMD,		// see sdk TrackPopupMenu
									point.x, point.y, 
									this);

	// Copy videos to a new location.
	if(nCmd  == ID_CTX_COPY_FILES)
	{
		CCopyFiles dlg;
		dlg.m_itemAr.Copy(itemsToCopy);
		if(dlg.DoModal() == IDOK)
		{
			SetFocus();

			// Deselect items if copied.
			if(dlg.m_nFilesCopied)
			{
				int i;
				POSITION pos = GetFirstSelectedItemPosition();
				while(pos)
				{
					i = GetNextSelectedItem(pos);
					SetItemState(i, 0, LVIS_SELECTED | LVIS_FOCUSED);
				}
			}
		}
	}

	// Delete videos.
	else if(nCmd  == ID_CTX_DELETE_FILES)
	{
		// Verify
		if(sz >1)
			str.Format(_T("Permanently delete %d selected videos?"), sz);
		else
			str = _T("Permanently delete selected video?");

		int ret = MessageBox(str, _T("Khan Desktop"), 
										MB_OKCANCEL | MB_ICONINFORMATION);
		// User canceled.
		if(ret == IDCANCEL)
			return;

		// Delete.
		//
		// Get directories.
		CString dir = pDoc->m_videoStorePath;
		dir += _T("\\");
		CString path;
		int nDeleted = 0;
		for(int i = 0; i < sz; i++)
		{
			path = dir;
			path += itemsToCopy[i].m_fileName;

			// Attempt copy.
			if(DeleteFile(path))
				nDeleted++;

			// Error
			else
			{
				CString msg;
				DWORD err = GetLastError();

				// Format last error.
				LPVOID lpMsgBuf;
				FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
							  FORMAT_MESSAGE_FROM_SYSTEM	 |
							  FORMAT_MESSAGE_IGNORE_INSERTS,
							  NULL,
							  err,
							  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
							  (LPTSTR) &lpMsgBuf,
							  0,
							  NULL);

				// Format err msg.
				msg.Format(_T("Video File deletion failed.\n") 
						   _T("%d of %d selected files deleted.\n%s"),
								nDeleted, sz, lpMsgBuf);
				msg.TrimRight();

				// Clean up.
				LocalFree(lpMsgBuf);

				// Info user.
				MessageBox(msg, _T("Khan Desktop"), MB_OK | MB_ICONINFORMATION);

				// Break out.
				break;
			}
		}
		// If any files deleted, rebuild list.
		if(nDeleted)
		{
			// Reload files.
			if(!LoadVideoFiles())
			{
				// If none, set an empty list and bail.
				DeleteAllItems();
				SetColumnWidth(0, 150);
				SetColumnWidth(1, 250);
				SetColumnWidth(2, 100);
				return;
			}
			// Reset col widths based on text extent.
			int c1, c2, c3;
			GetColWidths(c1, c2, c3);
			SetColumnWidth(0, c1 + 10);
			SetColumnWidth(1, c2 + 10);
			SetColumnWidth(2, c3 + 10);

			// Add video files.
			SetFilesList();

			// Set cursel video, if any.
			SetCurSelVideo();
		}
	}
	// Close List Files.
	else if(nCmd  == ID_CTX_CLOSE_FILES)
	{
		// Simulate clicking the Tools menu item ID_TOOLS_LIST_FILES.
		// Calls CDoc::OnToolsListFiles().
		AfxGetMainWnd()->PostMessage(WM_COMMAND, ID_TOOLS_LIST_FILES, 0);
	}
}

// Owner Draw list items.
// This resolves flicker and selection / focus 
// issues - for unknown reasons.
void CListFiles::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	if(lpDrawItemStruct->CtlType != ODT_LISTVIEW)
		return;

	CDC dc;
	dc.Attach(lpDrawItemStruct->hDC);
	dc.GetSafeHdc();
	if(!dc)
		return;

	CString strText;
	CRect rcItem, rcText;
	BOOL selected = FALSE;
	BOOL focused = FALSE;

	// Save these value to restore them when done drawing.
	COLORREF textColor		= dc.GetTextColor();
	COLORREF bkColor		= dc.GetBkColor();
	COLORREF hlghtTextColor = ::GetSysColor(COLOR_HIGHLIGHTTEXT);
	COLORREF hlghtBkColor	= ::GetSysColor(COLOR_HIGHLIGHT);

	// Set flag if selected.
	if ( (lpDrawItemStruct->itemAction | ODA_SELECT)  > 0 &&
		 (lpDrawItemStruct->itemState & ODS_SELECTED) > 0 )
	{
		selected = TRUE;;
	}

	// If this item has the focus, draw a frame around the
	// item's rect.
	if ( (lpDrawItemStruct->itemAction | ODA_FOCUS) > 0 &&
		 (lpDrawItemStruct->itemState & ODS_FOCUS) > 0  )
	{
		focused = TRUE; // not used
	}

	// Draw the text.
	int nCols = GetHeaderCtrl()->GetItemCount();
	for(int i = 0; i < nCols; i++)
	{
		strText = GetItemText(lpDrawItemStruct->itemID, i);
		GetSubItemRect(lpDrawItemStruct->itemID, i, LVIR_LABEL, rcItem);
		rcItem.left += 5;

		// Draw selected first col.
		if(selected && i == 0)
		{
			CSize sz = dc.GetTextExtent(strText);
			int colWidth = rcItem.Width();

			// If col is wider than text, highlight text width bkg.
			if(sz.cx < colWidth)
			{
				// Errase full col width bkg.
				dc.FillSolidRect(&rcItem, bkColor);
				// Highlight text width bkg only.
				rcText = rcItem;
				rcText.right = rcText.left + sz.cx;
				dc.FillSolidRect(&rcText, hlghtBkColor);
			}
			// Else highlight full col width bkg.
			else
				dc.FillSolidRect(&rcItem, hlghtBkColor);

			// Draw highlighted text.
			dc.SetTextColor(hlghtTextColor);
			dc.SetBkColor(hlghtBkColor);
			dc.DrawText(strText,
						strText.GetLength(),
						&rcItem,
						DT_LEFT | DT_SINGLELINE | DT_VCENTER);
		}

		// Draw normal.
		else
		{
			dc.FillSolidRect(&rcItem, bkColor);
			dc.SetTextColor(textColor);
			dc.SetBkColor(bkColor);
			dc.DrawText(strText,
						strText.GetLength(),
						&rcItem,
						DT_LEFT | DT_SINGLELINE | DT_VCENTER);
		}
	}

	// Clean up.
	dc.SetTextColor(textColor);
	dc.SetBkColor(bkColor);
	dc.Detach();
}

// Selecting - left mouse click special handling.
void CListFiles::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// def handling
	//CListCtrl::OnLButtonDown(nFlags, point);
	TRACE(_T("Entering CListFiles OnLButtonDown\n"));

	LVHITTESTINFO lvhti;
	lvhti.pt = point;
	SubItemHitTest(&lvhti);

	// If on left item text. do default.
	if (lvhti.flags & LVHT_ONITEMLABEL &&
		lvhti.iSubItem == 0)
	{
		CListCtrl::OnLButtonDown(nFlags, point);
	}
	// Else deselect all.
	else
	{
		int i;
		POSITION pos = GetFirstSelectedItemPosition();
		while(pos)
		{
			i = GetNextSelectedItem(pos);
			SetItemState(i, 0, LVIS_SELECTED);
		}
	}
}

// Context Menu - right click special handling.
void CListFiles::OnRButtonDown(UINT nFlags, CPoint point) 
{
	TRACE(_T("Entering CListFiles OnRButtonDown\n"));

	LVHITTESTINFO lvhti;
	lvhti.pt = point;
	SubItemHitTest(&lvhti);

	// If on a selected left item text. do default.
	if (lvhti.flags & LVHT_ONITEMLABEL &&
		lvhti.iSubItem == 0 &&
		lvhti.iItem >= 0)
	{
		// If selected.
		if(GetItemState(lvhti.iItem, LVIS_SELECTED) == LVIS_SELECTED)
		{
			CListCtrl::OnRButtonDown(nFlags, point);
			return;
		}
	}

	// Else deselect all, and pass directly
	// to OnContextMenuthen() for close list
	// context menu option.
	int i;
	POSITION pos = GetFirstSelectedItemPosition();
	while(pos)
	{
		i = GetNextSelectedItem(pos);
		SetItemState(i, 0, LVIS_SELECTED);
	}
	ClientToScreen(&point);
	OnContextMenu(this, point);
}


///////////////////////////////////////////////////////////////////
// CListItem
CListItem::CListItem()
{
}
CListItem::~CListItem()
{
}
CListItem::CListItem(const CListItem & item) 
{ 
	Copy(item); 
}
void CListItem::operator=(const CListItem & item) 
{ 
	Copy(item); 
};
BOOL CListItem::operator==(const CListItem & item)
{
	if(m_fileName == item.m_fileName &&
	   m_title == item.m_title &&
	   m_sizeKb == item.m_sizeKb)
		return TRUE;
	return FALSE;
}
BOOL CListItem::operator!=(const CListItem & item) 
{ 
	return !(*this == item); 
}
void CListItem::Copy(const CListItem & item)
{
	m_fileName = item.m_fileName;
	m_title = item.m_title;
	m_sizeKb = item.m_sizeKb;
}	
void CListItem::Zero() 
{
	m_fileName.Empty();
	m_title.Empty();
	m_sizeKb = 0;
}

////////////////////////////////////////////////////////////////
// Class CItemArray
//
// Sort by file names.
void CItemArray::SortNames(BOOL ascend)
{
	if(ascend)
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareNameAscend);
	else
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareNameDescend);
}

// Sort by titles.
void CItemArray::SortTitles(BOOL ascend) 
{ 
	if(ascend)
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareTitlesAscend);
	else
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareTitlesDescend);
}

// Sort by sizes.
void CItemArray::SortSizes(BOOL ascend)
{ 
	if(ascend)
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareSizeAscend);
	else
		qsort(GetData(), GetSize(), sizeof(CListItem), CompareSizeDescend);
}

// Compare file names.
int CItemArray::CompareNameAscend(const void *arg1, const void *arg2) 
{
	// Get the CListItems.
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	return item1->m_fileName.CompareNoCase(item2->m_fileName);
}
int CItemArray::CompareNameDescend(const void *arg1, const void *arg2) 
{
	// Get the CListItems.
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	return item2->m_fileName.CompareNoCase(item1->m_fileName);
}

// Compare titles.
int CItemArray::CompareTitlesAscend(const void *arg1, const void *arg2) 
{
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	return item1->m_title.CompareNoCase(item2->m_title);
}
int CItemArray::CompareTitlesDescend(const void *arg1, const void *arg2) 
{
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	return item2->m_title.CompareNoCase(item1->m_title);
}

// Compare sizes.
int CItemArray::CompareSizeAscend(const void *arg1, const void *arg2) 
{
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	// Lesser.
	if(item1->m_sizeKb < item2->m_sizeKb)
		return -1;
	// Greater.
	else if(item1->m_sizeKb > item2->m_sizeKb)
		return +1;
	// Equal.
	return 0;
}
int CItemArray::CompareSizeDescend(const void *arg1, const void *arg2) 
{
	CListItem * item1 = (CListItem *)arg1;
	CListItem * item2 = (CListItem *)arg2;
	// Lesser.
	if(item1->m_sizeKb > item2->m_sizeKb)
		return -1;
	// Greater.
	else if(item1->m_sizeKb < item2->m_sizeKb)
		return +1;
	// Equal.
	return 0;
}



