// SearchDialog.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "SearchDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// 10/04/2017 search
/////////////////////////////////////////////////////////////////////////////
// CSearchDialog dialog


CSearchDialog::CSearchDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CSearchDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSearchDialog)
	m_strStatic = _T("");
	m_strSearchEdit = _T("");
	m_strStaticResults = _T("");
	//}}AFX_DATA_INIT

	// Scrolling
	m_nDlgHeight	= 0;	// Workspace height in pixels
	m_nVScrollPos	= 0;	// Vertical scroll position
	m_nVPageSize	= 0;	// Vertical page size

	m_nSearched		= 0;	// used in GetSearchCurSelText()
	m_nHits			= 0;


	TRACE(_T("constructing CSearchDialog\n"));
}


void CSearchDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSearchDialog)
	DDX_Control(pDX, IDC_SEARCH_GO_BTN, m_ctrlBtnGoTo);
	DDX_Control(pDX, IDC_SEARCH_LIST, m_ctrlSearchList);
	DDX_Control(pDX, IDC_SEARCH_BUTTON, m_ctrlSearchBtn);
	DDX_Control(pDX, IDC_SEARCH_EDIT, m_ctrlSearchEdit);
	DDX_Text(pDX, IDC_SEARCH_STATIC, m_strStatic);
	DDX_Text(pDX, IDC_SEARCH_EDIT, m_strSearchEdit);
	DDX_Text(pDX, IDC_SEARCH_STATIC_RESULTS, m_strStaticResults);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSearchDialog, CDialog)
	//{{AFX_MSG_MAP(CSearchDialog)
	ON_WM_SIZE()
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_SEARCH_BUTTON, OnSearchButton)
	ON_LBN_SELCHANGE(IDC_SEARCH_LIST, OnSelchangeSearchList)
	ON_BN_CLICKED(IDC_SEARCH_GO_BTN, OnSearchGoBtn)
	ON_BN_CLICKED(IDC_SEARCH_CLEAR, OnSearchClear)
	//}}AFX_MSG_MAP

	// Updates.
	//ON_UPDATE_COMMAND_UI(IDC_SEARCH_GO_BTN, OnUpdateGoTo)

END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSearchDialog message handlers

BOOL CSearchDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;

	// Set list font same as contents tab
	if(pDoc)
	{
		LOGFONT lf = pDoc->m_treeLf;
		m_searchListFont.CreateFontIndirect(&lf);
		m_ctrlSearchList.SetFont(&m_searchListFont);
	}


	// Set CApp handle.
	((CApp *)AfxGetApp())->m_hwndSearchDlg = m_hWnd;

	m_strStatic = _T("Enter word(s) to search for.");
	m_strStaticResults = _T("Search results");
	UpdateData(FALSE);

	// Default right pane text - see CTabView::TabSelChange().
	m_strRightPaneDef = _T("Search Tab:\n\n") 
						_T("\tSearch for a single word or phrase you want to find.")
						_T(" Enclose phrases in double quotes.\n\n")
						_T("\tExample searches:\n")
						_T("\t  Algebra\n")
						_T("\t  \"linear algebra\"\n")
						_T("\t  root\n")
						_T("\t  \"Square Root\"\n")
						_T("\t  \"Leonardo da Vinci\"\n\n")
						_T("\tSearches are not case-sensitive.\n\n") 
						_T("\tPress the Search button or press enter to begin the search.\n\n") 
						_T("\tResults are limited to the first 2000 items found.\n\n") 
						_T("\tOnly Topic and Video titles/descriptions are searched.")
						_T(" Video voice transcriptions are not available or searched.\n\n"); 


	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSearchDialog::OnSize(UINT nType, int cx, int cy) 
{
	TRACE(_T("In CSearchDialog OnSize\n"));
	int nVScrollMax = 0;

	// Get dialog height from app - initialized in CTabView::CreatePages.
	m_nDlgHeight = ((CApp*)AfxGetApp())->m_searchDlgRect.Height();

	// Initialize and update our scrollinfo for vert bar.
	//
	// If dialog is larger than window, adjust bars
	if (cy < m_nDlgHeight) 
	{
		nVScrollMax = m_nDlgHeight - 1;
		m_nVPageSize = cy;
	}

	// Else, effectively hides bars
	else
	{
		// no scroll bars are needed. Position the dlg back to normal
		ScrollWindow(0, m_nVScrollPos);
		// init scrollinfo
		m_nVScrollPos = m_nVPageSize = 0;
	}

	SCROLLINFO si;
	si.fMask = SIF_PAGE | SIF_RANGE | SIF_POS;
	si.nMin = 0;
	si.nMax = nVScrollMax;
	si.nPos = m_nVScrollPos;
	si.nPage = m_nVPageSize;

	SetScrollInfo (SB_VERT, &si, TRUE);

	// dlg does not repaint after update. This forces the issue.
	Invalidate();
	if(si.nPos != 0)
		PostMessage(WM_VSCROLL);

	CDialog::OnSize(nType, cx, cy);	
}

// Vertical scroll
void CSearchDialog::OnVScroll(UINT nCode, UINT nPos, CScrollBar* pScrollBar) 
{
    int nDelta;

    // Compute the vertical scroll distance, or "delta."
    switch (nCode) 
	{

    case SB_LINEUP:
        nDelta = -LINESIZE;
        break;

    case SB_PAGEUP:
        nDelta = -m_nVPageSize;
        break;

    case SB_THUMBTRACK:
        nDelta = (int) nPos - m_nVScrollPos;
        break;

    case SB_PAGEDOWN:
        nDelta = m_nVPageSize;
        break;

    case SB_LINEDOWN:
        nDelta = LINESIZE;
        break;

    default: // Ignore other scroll bar messages
        return;
    }

    // Adjust the delta if adding it to the current scroll position would
    // cause an underrun or overrun.
    int nScrollPos = m_nVScrollPos + nDelta;
    int nMaxPos = m_nDlgHeight - m_nVPageSize;

    if (nScrollPos < 0)
        nDelta = -m_nVScrollPos;
    else if (nScrollPos > nMaxPos)
        nDelta = nMaxPos - m_nVScrollPos;

    // Update the scroll position and scroll the window.
    if (nDelta != 0) 
	{
        m_nVScrollPos += nDelta;
        SetScrollPos (SB_VERT, m_nVScrollPos, TRUE);
        ScrollWindow(0, -nDelta);
    }
	CDialog::OnVScroll(nCode, nPos, pScrollBar);
}

// Serch pressed.
void CSearchDialog::OnSearchButton() 
{
	// Flag to organize the many ways to fail. 
	BOOL isError = FALSE;

	// For m_strStaticResults static.
	CString strStaticResults = _T("Program error");

	// Get the search text.
	UpdateData(TRUE);

	// Copy
	CString strEdit = m_strSearchEdit;

	// The search btn was pressed - clean existing. 
	// results no matter what.
	m_results.RemoveAll();
	m_ctrlSearchList.ResetContent();
	m_nSearched = 0;		
	m_nHits = 0;


	////////////////
	// Parse input.
	CString term;
	CStringArray strAr;
	int len = strEdit.GetLength();
	BOOL inString = FALSE;

	for(int i = 0; i < len; i++)
	{
		// If a double quote.
		if(strEdit[i] == _T('"'))
		{
			if(!inString)
			{
				inString = TRUE;
				continue;
			}
			else
			{
				// Add string 
				if(!term.IsEmpty())
				{
					if(IsValidSearchTerm(term))
						strAr.Add(term);
					term = _T("");
				}
				inString = FALSE;
				continue;
			}
		}

		// If a space.
		if(!inString && strEdit[i] == _T(' '))
		{
			if(!term.IsEmpty())
			{
				if(IsValidSearchTerm(term))
					strAr.Add(term);
				term = _T("");
			}
			continue;
		}
		// If the last char
		if(i == len - 1 && !term.IsEmpty())
		{
			term += strEdit[i];
			if(IsValidSearchTerm(term))
				strAr.Add(term);
			continue;
		}
		// Add the char.
		term += strEdit[i];
	}

	///////////////////////////
	// Verify input & CDoc ptr.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	int strArSz = strAr.GetSize();
	if(!strArSz || strAr[0].IsEmpty() || !pDoc )
	{
		if(!pDoc)
			strStaticResults = _T("Program error");
		else
			strStaticResults = _T("Not searchable");
		isError = TRUE;
	}

	///////////////////////////////
	// If words/strings to search.
	int nSearched = 0, nHits = 0;
	if(!isError)
	{
		// Do the search.
		//
		CString term = strAr[0];

		// Get tree ref.
		CTopicTree & tree = pDoc->GetTree();

		// Do the search.
		BOOL ret = tree.Search(term, m_results, nSearched, nHits);
		if(!ret || !nHits || !m_results.GetSize())
		{
			strStaticResults = _T("Nothing found");
			isError = TRUE;
		}
	}

	// Allocate list.
	if(!isError)
	{
		// Allocate list.
		int nAlloc = m_ctrlSearchList.InitStorage(nHits + 2, 20);

		// If search and list alloc successfull.
		if(nAlloc == LB_ERRSPACE)
		{
			strStaticResults = _T("Program error");
			isError = TRUE;
		}
	}
		
	// Populate the list.
	if(!isError)
	{
		// Get tree ref.
		CTopicTree & tree = pDoc->GetTree();

		// Get a Dc for str length with font used in OnInitDialog()
		CDC * pDC = m_ctrlSearchList.GetDC();
		CFont * old = pDC->SelectObject(&m_searchListFont);

		// Use a tab to simulate a column for prefix.
		CString dash = _T("_"); // a bit wider than - 
		CSize dashSz = pDC->GetTextExtent(dash);
		int dlgUnits = (dashSz.cx * 4 / 3 * 4) / LOWORD(::GetDialogBaseUnits());
		m_ctrlSearchList.SetTabStops(dlgUnits);

		int maxLen = 0;
		CSize strSz;

		// Set strings.
		int nResults = m_results.GetSize();
		CString strPath, strDisp;
		for(int i = 0; i < nResults; i++)
		{
			// Get tree nodes made from topic/video Titles.
			// Truncate nodes to 15 chars.
			tree.SearchGetNodeTitles(m_results[i], strPath, 15);

			// If a video, prefix with a space.
			if(m_results[i].m_isVideo)
				strDisp = _T("\t");
			// Else preface Topic with a dash.
			else
				strDisp = _T("-\t");
			// Add the path.
			strDisp += strPath;

			m_ctrlSearchList.AddString(strDisp);

			// Get max len.
			strSz = pDC->GetTextExtent(strDisp);
			if(strSz.cx > maxLen)
				maxLen = strSz.cx;
		}
		// Set width + 1 char
		strSz = pDC->GetTextExtent(_T("X"));
		m_ctrlSearchList.SetHorizontalExtent(maxLen + strSz.cx);

		// Clean up.
		pDC->SelectObject(old);
		m_ctrlSearchList.ReleaseDC(pDC);

		// Set results str.
		m_strStaticResults.Format(_T("%d found"), nResults);
		UpdateData(FALSE);

		// Set stats used in GetSearchCurSelText()
		m_nSearched = nSearched;		
		m_nHits = nHits;

		// Set right pane text.
		CString str;
		str.Format(_T("Search Results:\n\n")
				   _T("\tClick a search item in the list to display the topic or")
				   _T(" video text.\n\n")
				   _T("\tItems prefaced with a dash are for Topic text, otherwise")
				   _T(" for Video text.\n\n")
				   _T("\tItems are abbreviated titles representing the Topic or")
				   _T(" Video's Contents Tab path.\n\n")
				   _T("\tPress the Go To button to show a selected item in the Contents Tab.\n\n")
				   _T("\tPress Clear button to empty the results list and search box.\n\n")
				   _T("\t%d title and description pairs searched.\n")
				   _T("\t%d matches found.\n"), nSearched, nHits); 
		pDoc->m_rawStr = str;

		// Update for text and close list files if open.
		pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);
	}

	// Error - one of:
	//	No search results
	//	Term not searchable
	//	Bad list alloc, no pDoc, 
	// 
	else
	{
		// Set results str with error msg from above.
		m_strStaticResults = strStaticResults;
		UpdateData(FALSE);

		// Right pane text back to default - see CTabView::TabSelChange().
		pDoc->m_rawStr = m_strRightPaneDef;
		// Update for text and close list files if open.
		pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);
	}
}

// Clear button pressed - clear results list and edit.
void CSearchDialog::OnSearchClear() 
{
	CDoc * pDoc = ((CApp *)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// Clean all.
	m_results.RemoveAll();
	m_ctrlSearchList.ResetContent();
	m_nSearched = 0;		
	m_nHits = 0;
	m_strSearchEdit = _T("");
	m_strStatic = _T("Enter word(s) to search for.");
	m_strStaticResults = _T("Search results");
	UpdateData(FALSE);

	// Right pane text back to default - see CTabView::TabSelChange().
	pDoc->m_rawStr = m_strRightPaneDef;
	// Update for text and close list files if open.
	pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);	
}

// Do not search common words.
BOOL CSearchDialog::IsValidSearchTerm(CString str)
{
	str.MakeLower();
	int len = str.GetLength();

	// Single chars.
	if(len == 1)
		return FALSE;

	// Reserved
	else if(str == _T("an")			|| 
			str == _T("and")		||
			str == _T("as")			||
			str == _T("at")			||
			str == _T("be")			||
			str == _T("but")		||
			str == _T("by")			||
			str == _T("description")||	// khan
			str == _T("do")			||
			str == _T("for")		||
			str == _T("from")		||
			str == _T("have")		||
			str == _T("he")			||
			str == _T("in")			||
			str == _T("it")			||
			str == _T("not")		||
			str == _T("of")			||
			str == _T("on")			||
			str == _T("or")			||
			str == _T("she")		||
			str == _T("that")		||
			str == _T("the")		||
			str == _T("there")		||
			str == _T("they")		||
			str == _T("title")		||	// khan
			str == _T("this")		||
			str == _T("to")			||
			str == _T("topic")		||	// khan
			str == _T("we")			||
			str == _T("video")		||	// khan
			str == _T("which")		||
			str == _T("with")		||
			str == _T("you")	)
		return FALSE;

	// Ok
	return TRUE;
}

// Search List clicked on.
// Fetch and display item title, description and path.
void CSearchDialog::OnSelchangeSearchList() 
{
	int iSel = m_ctrlSearchList.GetCurSel();
	int nResults = m_results.GetSize();

	// Verify.
	if(iSel == LB_ERR || nResults == 0)
		return;

	if(iSel < nResults)
	{
		// Get CDoc.
		CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
		if(!pDoc)
			return;

		// Get tree ref.
		CTopicTree & tree = pDoc->GetTree();

		CString titleStr, descriptionStr, pathStr;
		BOOL isVid = m_results[iSel].m_isVideo;

		if(tree.SearchGetStrings(m_results[iSel], titleStr, descriptionStr) && 
		   tree.SearchGetNodeTitles(m_results[iSel], pathStr, 0)			)
		{
			CString str;

			// Set right pane text.
			//
			if(isVid)
			{
				str.Format(_T("Video Title:\n")
						   _T("\t%s\n\n")
						   _T("Video Description:\n")
						   _T("\t%s\n\n")
						   _T("Path:\n")
						   _T("\t%s\n"), titleStr, descriptionStr, pathStr); 
			}
			else
			{
				str.Format(_T("Topic Title:\n")
						   _T("\t%s\n\n")
						   _T("Topic Description:\n")
						   _T("\t%s\n\n")
						   _T("Path:\n")
						   _T("\t%s\n"), titleStr, descriptionStr, pathStr);  
			}
			// Draw in view's right pane.
			pDoc->m_rawStr = str;
			pDoc->UpdateAllViews(NULL, RAW_STRING_EX, NULL);
		}
	}	
}

// Get Search List current selection text.
// Same code as OnSelchangeSearchList() above.
// Called from CTabView::TabSelChange().
BOOL CSearchDialog::GetSearchCurSelText(CString & str) 
{
	// Zero param.
	str = _T("");

	int iSel = m_ctrlSearchList.GetCurSel();
	int nResults = m_results.GetSize();
	int nItems = m_ctrlSearchList.GetCount();

	// Must be results.
	if(!nResults || !nItems || (nResults != nItems) )
		return FALSE;

	if(iSel != LB_ERR && iSel < nResults)
	{
		// Get CDoc.
		CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
		if(!pDoc)
			return FALSE;
		// Get tree ref.
		CTopicTree & tree = pDoc->GetTree();

		CString titleStr, descriptionStr, pathStr;
		BOOL isVid = m_results[iSel].m_isVideo;

		if(tree.SearchGetStrings(m_results[iSel], titleStr, descriptionStr) && 
		   tree.SearchGetNodeTitles(m_results[iSel], pathStr, 0)			)
		{
			if(isVid)
			{
				str.Format(_T("Video Title:\n")
						   _T("\t%s\n\n")
						   _T("Video Description:\n")
						   _T("\t%s\n\n")
						   _T("Path:\n")
						   _T("\t%s\n"), titleStr, descriptionStr, pathStr); 
			}
			else
			{
				str.Format(_T("Topic Title:\n")
						   _T("\t%s\n\n")
						   _T("Topic Description:\n")
						   _T("\t%s\n\n")
						   _T("Path:\n")
						   _T("\t%s\n"), titleStr, descriptionStr, pathStr);  
			}			return TRUE;
		}
	}
	// If a search was made but no item selected.
	else if(nResults && nItems == nResults)
	{
		// Set right pane text.
		int nSearched = 0, nHits = 0;
		str.Format(_T("Search Results:\n\n")
				   _T("\tClick a search item in the list to display the topic or")
				   _T(" video text.\n\n")
				   _T("\tItems prefaced with a dash are for Topic text, otherwise")
				   _T(" for Video text.\n\n")
				   _T("\tItems are abbreviated titles representing the Topic or")
				   _T(" Video's Contents Tab path.\n\n")
				   _T("\tPress the Go To button to show a selected item in the Contents Tab.\n\n")
				   _T("\t%d title and description pairs searched.\n")
				   _T("\t%d matches found.\n"), m_nSearched, m_nHits); 
		return TRUE;
	}
	// No go
	return FALSE;
}

// User pressed go to contents button.
void CSearchDialog::OnSearchGoBtn() 
{
	// Check all bounds.
	int iSel = m_ctrlSearchList.GetCurSel();
	if(iSel == LB_ERR) // no sel, -1
		return;

	int nListItems = m_ctrlSearchList.GetCount();
	if(!nListItems)
		return;

	int nResults = m_results.GetSize();
	if(!nResults || iSel >= nResults || iSel >= nListItems || nResults != nListItems)
		return;

	// Get CDoc.
	CDoc * pDoc = ((CApp*)AfxGetApp())->m_pDoc;
	if(!pDoc)
		return;

	// GoTo the contents.
	//
	// Switch to Contents tab.
	HWND hWndTab = ((CApp *)AfxGetApp())->m_hwndTabView;
	if(hWndTab)
		::PostMessage(hWndTab, WM_USER_PAGE_SWITCH, 0, PAGE_CONTENTS); // Contents Tab

	// Set contents to search item.
	HWND hWndTree = ((CApp *)AfxGetApp())->m_hwndTreeDlg;
	if(hWndTree)
	{
		// Get tree ref.
		CTopicTree & tree = pDoc->GetTree();

		// Get the path to the topic/vid.
		CStringArray nodeList;
		int iVideo = -1;
		if(tree.SearchGetItemPath(m_results[iSel], nodeList, iVideo))
		{

			// Init CDoc members.
			pDoc->m_searchNodes.Copy(nodeList);
			pDoc->m_searchVideoIndex = iVideo;

			// Message Contents Tree dlg.
			::PostMessage(hWndTree, WM_USER_TREE_SET_SEARCH, 0, 0);
		}
	}



	int g = 0;
	
}

/* perhaps kickidle - search web - a handler in CDoc no fix
// WM_USER_DLG_UPDATE for empying lis
// GoTo button UI Update.
void CSearchDialog::OnUpdateGoTo(CCmdUI* pCmdUI)
{
	int iSel = m_ctrlSearchList.GetCurSel();
	if(iSel == LB_ERR) // no sel
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable(TRUE);
}*/
/*
Take a look at WM_IDLEUPDATECMDUI, possible 
from a SendMessageToDescendants():

SendMessageToDescendants(WM_IDLEUPDATECMDUI, (WPARAM)TRUE, 0, TRUE, TRUE);

Ruben 
*/

