// SplitterView.cpp : implementation file
//

#include "stdafx.h"
#include "Khan.h"
#include "SplitterView.h"

#include "TabView.h"
#include "DataView.h"
#include "MainFrm.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSplitterView

IMPLEMENT_DYNCREATE(CSplitterView, CView)

CSplitterView::CSplitterView()
{
	TRACE(_T("constructing CSplitterView\n"));

	// Left pane width initial value to 1.
	// See OnSize(). v0.9.4
	m_leftPaneWidth = 1;

	// Hide show left pane - restore width.
	m_prevLeftPaneWidth = 0;

}

CSplitterView::~CSplitterView()
{
	TRACE(_T("destructing CSplitterView\n"));
}


BEGIN_MESSAGE_MAP(CSplitterView, CView)
	//{{AFX_MSG_MAP(CSplitterView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP

	ON_MESSAGE(WM_USER_HIDE_LEFT_PANE, OnHideLeftPane)


END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSplitterView drawing

void CSplitterView::OnDraw(CDC* pDC)
{
//	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CSplitterView diagnostics

#ifdef _DEBUG
void CSplitterView::AssertValid() const
{
	CView::AssertValid();
}

void CSplitterView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSplitterView message handlers

int CSplitterView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	TRACE(_T("in CSplitterView::OnCreate()\n"));

	// Base first.
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Create the main splitter view. There are two panes (2 columns, 1 row).
	// The left pane is a tab view (prop sheet) , the right pane is the main view.
	m_wndSplitter.CreateStatic(this, 1, 2);
	CCreateContext * pContext = (CCreateContext *)lpCreateStruct->lpCreateParams;
	lpCreateStruct->style |= WS_OVERLAPPED;

	// Create left pane.
	m_wndSplitter.CreateView(0,0,RUNTIME_CLASS(CTabView), CSize(0,0), pContext);
	// Create right pane.
	m_wndSplitter.CreateView(0,1,RUNTIME_CLASS(CDataView), CSize(0,0), pContext);

	// Set CApp handle
	((CApp*)AfxGetApp())->m_hwndSplitter = GetSafeHwnd();

	return 0; // OnCreate must return 0 to continue the creation of the CWnd object	
}

// Initial or new size.
void CSplitterView::OnSize(UINT nType, int cx, int cy) 
{
	TRACE(_T("in CSplitterView::OnSize()\n"));

	// Base class first
	CView::OnSize(nType, cx, cy);
	
	// Must call to establish window.
	m_wndSplitter.MoveWindow(0, 0, cx, cy);

	// Set initial left pane width to Download Dlg width.
	//
	// v0.9.4 - Set to 1 in constructor to fix hidden
	// pane (size = 0) calling here when resizing main 
	// window. See OnHideLeftPane() below.
	if(m_leftPaneWidth == 1)
	{
		m_leftPaneWidth = ((CApp*)AfxGetApp())->m_downloadDlgRect.Width();

		// If dlg width not available.
		if(m_leftPaneWidth == 1)
		{
			// Ballpark.
			DWORD units = GetDialogBaseUnits();
			m_leftPaneWidth  = LOWORD(units) * 34;
		}
	}
	// Set subsequent widths to current (user can drag).
	else
	{
		int i;
		m_wndSplitter.GetColumnInfo(0, m_leftPaneWidth, i);
	}

	// Set the Splitter Window columns.
	m_wndSplitter.SetColumnInfo(0, m_leftPaneWidth, 0);	// pxls

	// Recalc for new size.
	m_wndSplitter.RecalcLayout();	
}

// Start up.
void CSplitterView::OnInitialUpdate() 
{
	TRACE(_T("in CSplitterView::OnInitialUpdate()\n"));
	
	// Init views
	for(int i = 0; i < 2; i++) // 2 vertical panes 
	{
		CView* pView = (CView*)m_wndSplitter.GetPane(0, i);
		pView->OnInitialUpdate();
	}	
}

// Typically, splitter panes are hidden by destroying 
// and creating windows. Here we cheat and just 
// slide it over. Note user can still grab the hidden
// pane.
LRESULT CSplitterView::OnHideLeftPane(WPARAM wParam, LPARAM lParam)	// WM_USER_HIDE_LEFT_PANE
{
	// Hide.
	if(lParam)
	{
		// Save current width to member.
		int i;
		m_wndSplitter.GetColumnInfo(0, m_prevLeftPaneWidth, i);

		// Set left pane to 0 pix wide.
		m_wndSplitter.SetColumnInfo(0, 0, 0);

		// Recalc for new size.
		m_wndSplitter.RecalcLayout();	
	}
	else
	{
		// Get previous width.
		int width = m_prevLeftPaneWidth;
		if(width < 150)
			width = 150;

		// Set the left pane to previous.
		m_wndSplitter.SetColumnInfo(0, width, 0);

		// Recalc for new size.
		m_wndSplitter.RecalcLayout();
	}

	// Set focus to View to enable Mouse Wheel
	// and Space Bar handling.
	CView* pView = (CView*)m_wndSplitter.GetPane(0, 1);
	pView->SetFocus();

	return 0;
}



